let totalImages = 130;
let imgElement = document.getElementById("scrollImage");

// Preload critical images using <link rel="preload>
function preloadCriticalImages() {
    const head = document.getElementsByTagName('head')[0];
    for (let i = 1; i <= 10; i++) {
        let link = document.createElement('link');
        link.rel = 'preload';
        link.as = 'image';
        link.href = `./assets/images/hero-animation/img-${i}.webp`;
        head.appendChild(link);
    }
}

// Lazy load images using IntersectionObserver
function lazyLoadImages() {
    let observer = new IntersectionObserver((entries, observer) => {
        entries.forEach(entry => {
            if (entry.isIntersecting) {
                for (let i = 11; i <= totalImages; i++) {
                    const img = new Image();
                    img.src = `./assets/images/hero-animation/img-${i}.webp`;
                }
                observer.disconnect(); // Stop observing after all images are loaded
            }
        });
    });
    observer.observe(document.querySelector('.hero-section'));
}

// Preload only the first few images eagerly, lazy load the rest
document.addEventListener("DOMContentLoaded", function () {
    preloadCriticalImages(); // Preload first few images

    if (window.innerWidth >= 768) {
        initializeIcons();
        addScrollBasedAnimation(false);
    } else {
        addScrollBasedAnimation(true);
    }

    // Lazy load images when the main thread is idle
    window.requestIdleCallback(() => {
        lazyLoadImages();
    });
});

// Initialize icons with no layout shift
function initializeIcons() {
    const icons = document.querySelectorAll('.hero-section img:not(#scrollImage)');
    const heading = document.querySelector('.icons-text');

    gsap.set([icons, heading], {
        x: 0,
        y: 0,
        opacity: 1,
        scale: 1,
        position: 'absolute',
        willChange: 'transform, opacity'
    });
    gsap.set('.primebook-after', { opacity: 0, willChange: 'opacity' });
}

function addScrollBasedAnimation(isMobile) {
    const icons = document.querySelectorAll('.hero-section img:not(#scrollImage)');
    const heading = document.querySelector('.icons-text');
    const imgContainer = document.querySelector('.image-change-box');
    const finalSection = document.querySelector('.primebook-after');
    const heroDiv = document.querySelector('.heroDiv');
    const triggerElement = isMobile ? ".heroDiv" : ".heroDiv";
    const startTrigger = isMobile ? "top +6%" : "top +11%";

    const floatingAnimation = gsap.to(icons, {
        y: gsap.utils.random(-10, 10, true),
        x: gsap.utils.random(-10, 10, true),
        repeat: -1,
        yoyo: true,
        ease: "power1.inOut",
        duration: gsap.utils.random(2, 3, true),
        delay: gsap.utils.random(0, 0.5, true),
        willChange: 'transform',
        paused: true
    });

    floatingAnimation.play();
    gsap.registerPlugin(ScrollTrigger);

    const tl = gsap.timeline({
        scrollTrigger: {
            trigger: triggerElement,
            start: startTrigger,
            end: "+=200%",
            scrub: 1,
            pin: true,
            pinSpacing: true,
            anticipatePin: 1,
            fastScrollEnd: true,
            onEnter: () => floatingAnimation.pause(),
            onLeaveBack: () => floatingAnimation.play()
        }
    });

    // Animate icons and heading...
    icons.forEach(icon => {
        const iconCenterX = window.innerWidth / 2 - icon.clientWidth / 2;
        const iconCenterY = window.innerHeight / 2 - icon.clientHeight / 2;

        tl.to(icon, {
            x: iconCenterX - icon.getBoundingClientRect().left,
            y: iconCenterY - icon.getBoundingClientRect().top,
            scale: 0,
            opacity: 0,
            duration: 1,
            ease: "power1.inOut",
            willChange: 'transform, opacity'
        }, 0);
    });

    tl.to(heading, {
        scale: 0,
        opacity: 0,
        duration: 1,
        ease: "power1.inOut",
        willChange: 'transform, opacity'
    }, 0.1);

    tl.to(imgContainer, {
        opacity: 1,
        scale: 1,
        duration: 1,
        ease: "power1.inOut",
        willChange: 'transform, opacity'
    }, "-=0.4");

    // Scroll-triggered image changes
    gsap.to({}, {
        scrollTrigger: {
            trigger: imgContainer,
            start: "top top",
            end: `+=${totalImages * 5}px`,
            scrub: 5,
            onUpdate: (self) => {
                const progress = self.progress * (totalImages - 1);
                const imgIndex = Math.round(progress) + 1;
                imgElement.src = `./assets/images/hero-animation/img-${imgIndex}.webp`;
            }
        }
    });

    if (!isMobile) {
        tl.to(finalSection, {
            opacity: 1,
            duration: 1,
            ease: "power1.inOut",
            willChange: 'opacity'
        }, `+=0.5`);
    }

    tl.to('#pinnedsection', {
        y: "-60px",
        duration: 1,
        ease: "power1.inOut",
        delay: 0.5
    });
}

document.addEventListener("DOMContentLoaded", function () {
    const imageContainer = document.getElementById("gurkha-suv");
    const sliderThumb = document.getElementById("slider-thumb");
    const sliderValue = document.querySelector(".slider-value");
    const totalImages = 26; // Total number of images
    let currentImageIndex = 1; // Start with the first image (img-1.webp)
    let images = []; // Store preloaded images
    const cachedLastImageIndex = localStorage.getItem('lastImageIndex') || 1;

    // Hide image container initially
    imageContainer.style.visibility = 'hidden'; // Keep the image container hidden during preload

    // Preload images (but don't store them in localStorage)
    function preloadImages() {
        const promises = [];
        for (let i = 1; i <= totalImages; i++) {
            const img = new Image();
            img.src = `./assets/images/360-images/img-${i}.webp`;

            // Cache the images in memory (not localStorage)
            images.push(img);
            promises.push(
                new Promise((resolve) => {
                    img.onload = resolve; // Resolve the promise when the image is loaded
                })
            );
        }
        return Promise.all(promises); // Ensure all images are preloaded before continuing
    }

    // Function to update the displayed image
    function updateImage() {
        imageContainer.style.backgroundImage = `url(${images[currentImageIndex - 1].src})`;

        // Update slider value with degree symbol (HTML entity for the degree)
        const degreeValue = Math.round((currentImageIndex / totalImages) * 360);
        sliderValue.innerHTML = `${degreeValue}&deg;`; // Display the degree symbol
    }

    // Preload images and display the first image
    preloadImages().then(() => {
        imageContainer.style.visibility = 'visible'; // Make the image container visible after preload
        currentImageIndex = cachedLastImageIndex;
        updateImage(); // Show the cached last image or first image if none
    });

    // Set initial slider thumb position
    function setInitialSliderPosition() {
        sliderThumb.style.left = `0px`; // Start the slider at the leftmost position
        sliderValue.innerHTML = "15&deg;"; // Start at an initial value of 15 degrees
    }
    setInitialSliderPosition(); // Initialize the slider

    // Handle dragging for both mouse and touch events
    let isDragging = false;
    let startX = 0;

    // Start drag function (for both mouse and touch)
    function startDrag(e) {
        isDragging = true;
        startX = e.clientX || e.touches[0].clientX; // Get clientX for mouse or touch event
        e.preventDefault(); // Prevent default behavior like text selection
    }

    // Drag function
    function onDrag(e) {
        if (isDragging) {
            const clientX = e.clientX || e.touches[0].clientX; // Get clientX for mouse or touch event
            const sliderLine = document.querySelector(".slider-line");
            const sliderLineRect = sliderLine.getBoundingClientRect();
            const sliderLineWidth = sliderLine.clientWidth;

            let newLeft = clientX - sliderLineRect.left; // Calculate the thumb's new position

            // Constrain the thumb to stay within slider bounds
            if (newLeft < 0) newLeft = 0;
            if (newLeft > sliderLineWidth) newLeft = sliderLineWidth;

            // Update slider thumb position
            sliderThumb.style.left = `${newLeft}px`;

            // Calculate new image index based on slider thumb position
            const newImageIndex = Math.round((newLeft / sliderLineWidth) * (totalImages - 1)) + 1;
            if (newImageIndex !== currentImageIndex) {
                currentImageIndex = newImageIndex;
                updateImage(); // Update the displayed image
                localStorage.setItem('lastImageIndex', currentImageIndex); // Cache last image index
            }
        }
    }

    // Stop drag function (for both mouse and touch)
    function stopDrag() {
        isDragging = false;
    }

    // Add event listeners for drag and slider movement
    imageContainer.addEventListener("mousedown", startDrag);
    imageContainer.addEventListener("mousemove", onDrag);
    imageContainer.addEventListener("mouseup", stopDrag);
    imageContainer.addEventListener("mouseleave", stopDrag);

    // Add touch events for mobile drag
    imageContainer.addEventListener("touchstart", startDrag);
    imageContainer.addEventListener("touchmove", onDrag);
    imageContainer.addEventListener("touchend", stopDrag);

    // Add mouse and touch events for the slider thumb
    sliderThumb.addEventListener("mousedown", startDrag);
    sliderThumb.addEventListener("touchstart", startDrag);

    // Add global mouse and touch events to handle dragging
    document.addEventListener("mousemove", onDrag);
    document.addEventListener("mouseup", stopDrag);
    document.addEventListener("touchmove", onDrag);
    document.addEventListener("touchend", stopDrag);
});

