let totalImages = 130;
let imgElement = document.getElementById("scrollImage");

// Preload only the first few critical images
function preloadCriticalImages() {
    const head = document.getElementsByTagName('head')[0];
    const preloadCount = 2; // Only preload the first two images
    for (let i = 1; i <= preloadCount; i++) {
        let link = document.createElement('link');
        link.rel = 'preload';
        link.as = 'image';
        link.href = `./assets/images/hero-animation/img-${i}.webp`;
        head.appendChild(link);
    }
}

// Lazy load remaining images with IntersectionObserver and fallback
function lazyLoadImages() {
    const lazyImages = [];
    for (let i = 3; i <= totalImages; i++) {  // Start from the 3rd image for lazy loading
        lazyImages.push(`./assets/images/hero-animation/img-${i}.webp`);
    }

    let observer = new IntersectionObserver((entries) => {
        entries.forEach(entry => {
            if (entry.isIntersecting) {
                lazyImages.forEach((src) => {
                    const img = new Image();
                    img.src = src;
                });
                observer.disconnect(); // Stop observing once images are loaded
            }
        });
    }, { rootMargin: '200px 0px' });

    observer.observe(imgElement);
}

// Preload critical images, initialize animations, and lazy load others
document.addEventListener("DOMContentLoaded", function () {
    preloadCriticalImages(); // Preload only the first two images

    if (window.innerWidth >= 768) {
        initializeIcons();
        addScrollBasedAnimation(false);
    } else {
        addScrollBasedAnimation(true);
    }

    // Use requestIdleCallback to load additional images when the main thread is idle
    if ('requestIdleCallback' in window) {
        window.requestIdleCallback(() => lazyLoadImages());
    } else {
        setTimeout(() => lazyLoadImages(), 1000); // Fallback for browsers without requestIdleCallback
    }
});

// Initialize icons with no layout shift
function initializeIcons() {
    const icons = document.querySelectorAll('.hero-section img:not(#scrollImage)');
    const heading = document.querySelector('.icons-text');
    gsap.set([icons, heading], {
        x: 0,
        y: 0,
        opacity: 1,
        scale: 1,
        position: 'absolute',
        willChange: 'transform, opacity'
    });
    gsap.set('.primebook-after', { opacity: 0, willChange: 'opacity' });
}

// Scroll-based animation function
function addScrollBasedAnimation(isMobile) {
    const icons = document.querySelectorAll('.hero-section img:not(#scrollImage)');
    const heading = document.querySelector('.icons-text');
    const imgContainer = document.querySelector('.image-change-box');
    const finalSection = document.querySelector('.primebook-after');
    const triggerElement = ".heroDiv";
    const startTrigger = isMobile ? "top +3%" : "top +7%";
    const endMultiplier = isMobile ? 2.5 : 2;
    const endTrigger = `+=${window.innerHeight * endMultiplier - 280}px`;
    const scrubValue = isMobile ? 2 : 1;
    gsap.registerPlugin(ScrollTrigger);

    const tl = gsap.timeline({
        scrollTrigger: {
            trigger: triggerElement,
            start: startTrigger,
            end: endTrigger,
            scrub: scrubValue,
            pin: true,
            pinSpacing: true,
            anticipatePin: 1,
            fastScrollEnd: true,
            refreshPriority: 1,
            onEnter: () => floatingAnimation.pause(),
            onLeaveBack: () => floatingAnimation.play(),
            onUpdate: (self) => {
                requestAnimationFrame(() => updateImageOnScroll(self.progress));
            }
        }
    });

    const floatingAnimation = gsap.to(icons, {
        y: gsap.utils.random(-10, 10, true),
        x: gsap.utils.random(-10, 10, true),
        repeat: -1,
        yoyo: true,
        ease: "power1.inOut",
        duration: gsap.utils.random(2, 3, true),
        delay: gsap.utils.random(0, 0.5, true),
        willChange: 'transform',
        paused: true
    });

    floatingAnimation.play();

    icons.forEach(icon => {
        const iconCenterX = window.innerWidth / 2 - icon.clientWidth / 2;
        const iconCenterY = window.innerHeight / 2 - icon.clientHeight / 2;

        tl.to(icon, {
            x: iconCenterX - icon.getBoundingClientRect().left,
            y: iconCenterY - icon.getBoundingClientRect().top,
            scale: 0,
            opacity: 0,
            duration: 1,
            ease: "power1.inOut",
            willChange: 'transform, opacity'
        }, 0);
    });

    tl.to(heading, {
        scale: 0,
        opacity: 0,
        duration: 1,
        ease: "power1.inOut",
        willChange: 'transform, opacity'
    }, 0.1);

    tl.to(imgContainer, {
        opacity: 1,
        scale: 1,
        duration: 1,
        ease: "power1.inOut",
        willChange: 'transform, opacity'
    }, "-=0.4");

    function updateImageOnScroll(progress) {
        const imageIndex = Math.round(progress * (totalImages - 1)) + 1;
        const newSrc = `./assets/images/hero-animation/img-${imageIndex}.webp`;
        if (imgElement.src !== newSrc) {
            imgElement.src = newSrc;
        }
    }

    updateImageOnScroll(ScrollTrigger.normalizeScroll());

    if (!isMobile) {
        tl.to(finalSection, {
            opacity: 1,
            duration: 1,
            ease: "power1.inOut",
            willChange: 'opacity'
        }, `+=0.5`);
    } else {
        tl.to('.pin-spacer', {
            height: '290svh',
            ease: "none",
            delay: 0.5
        }, '+=1');
    }

    tl.to('#pinnedsection', {
        y: "-60px",
        duration: 1,
        ease: "power1.inOut",
        delay: 0.5
    });
}


document.addEventListener('DOMContentLoaded', function(){
    gsap.from('#VidCont', {
        y: '60px',
        ease: 'none',
        duration: 0.5,
        scrollTrigger: {
            trigger: '#VidCont',
            start: 'top 80%',  // Animation starts when the element enters 80% of the viewport
            end: 'top 20%',    // Animation ends when the element reaches 20% of the viewport
            toggleActions: 'play none none none',
        }
    });
});

    
document.addEventListener("DOMContentLoaded", function () {
    const imageContainer = document.getElementById("gurkha-suv");
    const sliderThumb = document.getElementById("slider-thumb");
    const sliderValue = document.querySelector(".slider-value");
    const totalImages = 26; // Total number of images
    let currentImageIndex = 1; // Start with the first image (img-1.webp)
    let images = []; // Store preloaded images
    const cachedLastImageIndex = localStorage.getItem('lastImageIndex') || 1;

    // Hide image container initially
    imageContainer.style.visibility = 'hidden'; // Keep the image container hidden during preload

    // Preload images (but don't store them in localStorage)
    function preloadImages() {
        const promises = [];
        for (let i = 1; i <= totalImages; i++) {
            const img = new Image();
            img.src = `./assets/images/360-images/img-${i}.webp`;

            // Cache the images in memory (not localStorage)
            images.push(img);
            promises.push(
                new Promise((resolve) => {
                    img.onload = resolve; // Resolve the promise when the image is loaded
                })
            );
        }
        return Promise.all(promises); // Ensure all images are preloaded before continuing
    }

    // Function to update the displayed image
    function updateImage() {
        imageContainer.style.backgroundImage = `url(${images[currentImageIndex - 1].src})`;

        // Update slider value with degree symbol (HTML entity for the degree)
        const degreeValue = Math.round((currentImageIndex / totalImages) * 360);
        sliderValue.innerHTML = `${degreeValue}&deg;`; // Display the degree symbol
    }

    // Preload images and display the first image
    preloadImages().then(() => {
        imageContainer.style.visibility = 'visible'; // Make the image container visible after preload
        currentImageIndex = cachedLastImageIndex;
        updateImage(); // Show the cached last image or first image if none
    });

    // Set initial slider thumb position
    function setInitialSliderPosition() {
        sliderThumb.style.left = `0px`; // Start the slider at the leftmost position
        sliderValue.innerHTML = "15&deg;"; // Start at an initial value of 15 degrees
    }
    setInitialSliderPosition(); // Initialize the slider

    // Handle dragging for both mouse and touch events
    let isDragging = false;
    let startX = 0;

    // Start drag function (for both mouse and touch)
    function startDrag(e) {
        isDragging = true;
        startX = e.clientX || e.touches[0].clientX; // Get clientX for mouse or touch event
        e.preventDefault(); // Prevent default behavior like text selection
    }

    // Drag function
    function onDrag(e) {
        if (isDragging) {
            const clientX = e.clientX || e.touches[0].clientX; // Get clientX for mouse or touch event
            const sliderLine = document.querySelector(".slider-line");
            const sliderLineRect = sliderLine.getBoundingClientRect();
            const sliderLineWidth = sliderLine.clientWidth;

            let newLeft = clientX - sliderLineRect.left; // Calculate the thumb's new position

            // Constrain the thumb to stay within slider bounds
            if (newLeft < 0) newLeft = 0;
            if (newLeft > sliderLineWidth) newLeft = sliderLineWidth;

            // Update slider thumb position
            sliderThumb.style.left = `${newLeft}px`;

            // Calculate new image index based on slider thumb position
            const newImageIndex = Math.round((newLeft / sliderLineWidth) * (totalImages - 1)) + 1;
            if (newImageIndex !== currentImageIndex) {
                currentImageIndex = newImageIndex;
                updateImage(); // Update the displayed image
                localStorage.setItem('lastImageIndex', currentImageIndex); // Cache last image index
            }
        }
    }

    // Stop drag function (for both mouse and touch)
    function stopDrag() {
        isDragging = false;
    }

    // Add event listeners for drag and slider movement
    imageContainer.addEventListener("mousedown", startDrag);
    imageContainer.addEventListener("mousemove", onDrag);
    imageContainer.addEventListener("mouseup", stopDrag);
    imageContainer.addEventListener("mouseleave", stopDrag);

    // Add touch events for mobile drag
    imageContainer.addEventListener("touchstart", startDrag);
    imageContainer.addEventListener("touchmove", onDrag);
    imageContainer.addEventListener("touchend", stopDrag);

    // Add mouse and touch events for the slider thumb
    sliderThumb.addEventListener("mousedown", startDrag);
    sliderThumb.addEventListener("touchstart", startDrag);

    // Add global mouse and touch events to handle dragging
    document.addEventListener("mousemove", onDrag);
    document.addEventListener("mouseup", stopDrag);
    document.addEventListener("touchmove", onDrag);
    document.addEventListener("touchend", stopDrag);
});

